
from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.hazmat.primitives import serialization, hashes
#from PIL import Image
from stegano import lsb


def chunk_data(data, chunk_size):
    """Split data into chunks of a specific size."""
    return [data[i:i + chunk_size] for i in range(0, len(data), chunk_size)]

def encrypt_large_file(file_path, public_key_path):
    """Encrypt a large file in chunks using RSA."""
    # Load the public key
    with open(public_key_path, "rb") as key_file:
        public_key = serialization.load_pem_public_key(key_file.read())

    # Read the file content
    with open(file_path, "rb") as f:
        plaintext = f.read()

    # Chunk size must be smaller than the RSA key size minus padding
    chunk_size = 190  # For a 2048-bit RSA key with OAEP-SHA256
    chunks = chunk_data(plaintext, chunk_size)

    # Encrypt each chunk
    encrypted_chunks = [
        public_key.encrypt(
            chunk,
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None,
            ),
        )
        for chunk in chunks
    ]

    # Save the encrypted chunks to a file
    encrypted_file_path = f"{file_path}_encrypted.txt"
    with open(encrypted_file_path, "wb") as f:
        for chunk in encrypted_chunks:
            f.write(chunk)

    print(f"File encrypted and saved to {encrypted_file_path}")

def decrypt_large_file(encrypted_file_path, private_key_path):
    """Decrypt a large file in chunks using RSA."""
    # Load the private key
    with open(private_key_path, "rb") as key_file:
        private_key = serialization.load_pem_private_key(
            key_file.read(),
            password=None,
        )

    # Read the encrypted file content
    with open(encrypted_file_path, "rb") as f:
        encrypted_data = f.read()

    # Chunk size for decryption matches the RSA key size
    chunk_size = 256  # For a 2048-bit RSA key
    chunks = chunk_data(encrypted_data, chunk_size)

    # Decrypt each chunk
    decrypted_chunks = [
        private_key.decrypt(
            chunk,
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None,
            ),
        )
        for chunk in chunks
    ]

    # Combine decrypted chunks and save to a new file
    decrypted_file_path = encrypted_file_path.replace("encrypted.txt", "decrypted.docx")
    with open(decrypted_file_path, "wb") as f:
        f.write(b"".join(decrypted_chunks))

    print(f"File decrypted and saved to {decrypted_file_path}")


def generate_keys():
    """Generate RSA private and public keys."""
    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
    )
    public_key = private_key.public_key()

    # Save the private key to a file
    with open("private_key.pem", "wb") as private_file:
        private_file.write(
            private_key.private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.PKCS8,
                encryption_algorithm=serialization.NoEncryption(),
            )
        )

    # Save the public key to a file
    with open("public_key.pem", "wb") as public_file:
        public_file.write(
            public_key.public_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PublicFormat.SubjectPublicKeyInfo,
            )
        )

    print("Keys generated and saved to private_key.pem and public_key.pem")


def hide_private_key(image_path, rsa_private_key):
    with open(rsa_private_key, "rt") as key_file:
        private_key_data = key_file.read()

    secret = lsb.hide(image_path, private_key_data)
    secret.save('output_image_path.png')

    print(f"RSA Private Key successfully hidden in output_image_path ")

def extract_private_key(image_path):
    rsa_private_key = lsb.reveal(image_path)

    with open('output_key_path.pem', "wt") as key_file:
        key_file.write(rsa_private_key)
    return rsa_private_key

while True:
    print("RSA File Encryption/Decryption Tool")
    print("1. Generera RSA nycklar")
    print("2. Kryptera en fil")
    print("3. Dekryptera a fil")
    print("4. Göm en nyckel i en bild")
    print("5. Extrahera en nyckel från en bild")
    print("6. Avsluta")
    choice = input("Enter your choice: ")

    if choice == "1":
        generate_keys()
    elif choice == "2":
        file_path = input("Ange sökvägen till filen som ska krypteras: ")
        public_key_path = input("Ange sökvägen till publika nyckeln: ")
        encrypt_large_file(file_path, public_key_path)
    elif choice == "3":
        encrypted_file_path = input("Ange en sökväg till krypterade filen: ")
        private_key_path = input("Ange sökvägen till privata nyckeln: ")
        decrypt_large_file(encrypted_file_path, private_key_path)
    elif choice == "4":
        image_path = input("Ange sökvägen till bilden: ")
        private_key_path = input("Ange sökvägen till nyckeln: ")
       # output_image_path = input("Enter the path to save the output image: ")
        hide_private_key(image_path, private_key_path)

    elif choice == "5":
        image_path = input("Ange sökvägen till bilden med gömd nyckel i: ")
        #output_key_path = input("Enter the path to save the extracted key: ")
        #key_size = int(input("Enter the key size in bytes (e.g., 256 for 2048-bit RSA): "))
        key = extract_private_key(image_path)
        print(key)
    elif choice == "6":
        print("Avslutar...")
        break
